# if you don't use bash then set your first line to '#!/bin/bash'
#
# you will also need the 'vp_get' function written by the author of this 
# script (which can be found in .bash_fun; don't 
# also forget to export it using 'export -f vp_get' in your .bashrc or
# otherwise you will have to 'source $HOME/.bash_fun' every time you
# use this script)
# Author of this script: Artur Merke <artur.merke@udo.edu>
######################################################################

#user defined options

PATH_TO_DIRECTORY_IN_WHICH_THIS_SCRIPT_IS=$PWD

#the argument is just a default value, if you already defined $BS2K_DIR
#then this setting will not be changed by what follows
TEAMNAME=BS03

BS2K_DIR=${BS2K_DIR:=$PATH_TO_DIRECTORY_IN_WHICH_THIS_SCRIPT_IS/bs2k}
BS2K_OPT=${BS2K_OPT:="\
 -s_type 12\
 -view_type 22 -neck_type 42\
 -wm_save 0 -wm_test 1 -wm_use_pfilter 1"}

BS2K_EXEC="BS2kAgent"

COACH_DIR=${COACH_DIR:=$PATH_TO_DIRECTORY_IN_WHICH_THIS_SCRIPT_IS/sputcoach/}
COACH_EXEC=SputCoach
COACH_OPT="-synchmode 1 -conf \
      $PATH_TO_DIRECTORY_IN_WHICH_THIS_SCRIPT_IS/sputcoach/coach.conf"

######################################################################

USAGE_MESSAGE="
usage: $0
 -h             #this screen
 -help          #this screen
 -num     <int> #number of players, which should be in [1,...,10] 
 -g             #start a goalie
 -goalie        #start a goalie 
 -c             #start our coach
 -coach         #start our coach  

 -t_name        #name for agents and coach 
 -all           #start a goalie, 10 players and the coach

 -dummy         #use artbots (like 11Pythons but for rcssserver 8.X)

 <advanced options, but very useful>

 -kill          #kill your agents  
 -show_ports    #show all used ports by the agents
 -debug         #just show what would be done, but don't start any programs  

"
AGENT_DIR=$BS2K_DIR;
AGENT_OPT=$BS2K_OPT;
AGENT_EXEC=$BS2K_EXEC;

PRINT_USAGE=""
USE_GOALIE=""
USE_COACH=""
NUMBER=""
KILL=""
DEBUG=""
SHOW_PORTS=""

######################################################################
#reading options by using the valueparser!!!
VP_KEEP=$*
vp_get "h"          $VP_KEEP && PRINT_USAGE=1
vp_get "help"       $VP_KEEP && PRINT_USAGE=1
vp_get "num"        $VP_KEEP && NUMBER=$VP_RES
vp_get "g"          $VP_KEEP && USE_GOALIE=1
vp_get "goalie"     $VP_KEEP && USE_GOALIE=1
vp_get "c"          $VP_KEEP && USE_COACH=1
vp_get "coach"      $VP_KEEP && USE_COACH=1
vp_get "all"        $VP_KEEP && NUMBER=10 && USE_GOALIE=1 && USE_COACH=1
vp_get "t_name"     $VP_KEEP && TEAMNAME=$VP_RES
vp_get "kill"       $VP_KEEP && KILL=1 
vp_get "debug"      $VP_KEEP && DEBUG=1 
vp_get "show_ports" $VP_KEEP && SHOW_PORTS=1
vp_get "dummy"      $VP_KEEP;
if [ $VP_GOT ]
then
  echo 
fi

#reading of all options done (life can be so easy when you use the right tools ;-)

AGENT_OPT="$AGENT_OPT -t_name $TEAMNAME"
COACH_OPT="$COACH_OPT -teamName $TEAMNAME"
AGENT="$AGENT_EXEC $AGENT_OPT"
COACH="$COACH_EXEC $COACH_OPT"

echo -e "\nDEBUG=$DEBUG,NUMBER=$NUMBER,USE_GOALIE=$USE_GOALIE,USE_COACH=$USE_COACH,PRINT_USAGE=$PRINT_USAGE"

######################################################################
# kill agents if required
if [ $KILL ] #kill remaining clients
then
  echo "killing remaining clients"
  DUM="\ps axu | grep ${USER} | grep ${AGENT_EXEC} | gawk '{ print $2 }' | xargs kill -9"
  if [ $DEBUG ] 
  then 
    echo $DUM
  else 
    #just $DUM doesn't work, why?
    \ps axu | grep ${USER} | grep ${AGENT_EXEC} | gawk '{ print $2 }' | xargs kill -9
    [ $USE_COACH ] && \ps axu | grep ${USER} | grep ${COACH_EXEC} | gawk '{ print $2 }' | xargs kill -9
  fi
  exit
fi

######################################################################
# show used UPD ports of your agents
if [ $SHOW_PORTS ] #show used UPD ports of your agents
then
  echo "used ports:"
  DUM="lsof | grep $USER | grep $AGENT_EXEC | grep UDP"
  if [ $DEBUG ] 
  then 
    echo $DUM
  else 
    lsof | grep $USER | grep $AGENT_EXEC | grep UDP
    #just $DUM doesn't work, why?
  fi
  exit
fi

######################################################################
# print usage if required
if [ -n "$PRINT_USAGE" -o \
     -z "$USE_GOALIE" -a -z "$NUMBER" -a -z "$USE_COACH" ] 
then 
  echo -e "$USAGE_MESSAGE" 
  exit
fi

######################################################################
# checking if directory exists, some output
if [ ! -d $AGENT_DIR ]  #does the directory exist at all???
then
  echo "no such directory $AGENT_DIR, exit!"
  exit 
fi

echo -e "AGENT_DIR=$AGENT_DIR\n"
cd $AGENT_DIR

AGENT="$AGENT $VP_KEEP"  #don't loose remaining options
COACH="$COACH $VP_KEEP"  #don't loose remaining options

echo "agent: $AGENT"

######################################################################
# starting agents
if [ $USE_GOALIE ]
then
  echo "starting goalie"
  DUM="$AGENT -goalie -s_type 124" 
  if [ $DEBUG ]; then echo "$DUM &; sleep 1"; else $DUM & sleep 1; fi
fi

if [ $NUMBER ] 
then
  if [ $NUMBER -ge 11 ]; then echo "to much players ($NUMBER), reducing to 10"; NUMBER=10; fi
  for ((a=0;a<$NUMBER;a++)) 
  do
    echo "starting player $a"
    DUM=$AGENT
    if [ $DEBUG ]; then echo "$DUM &"; else $DUM & fi	
  done
fi

######################################################################
# starting coach
if [ $USE_COACH ]
then
  cd $COACH_DIR
  echo "starting coach"
  DUM=$COACH
  if [ $DEBUG ]; then echo "$DUM"; else $DUM & fi
fi


echo "done"

exit 0
